from typing import List, Optional
from unittest.mock import Mock, AsyncMock

import pytest

from arcadepy._client import Arcade, AsyncArcade
from arcadepy.types.shared import AuthorizationResponse
from arcadepy.resources.auth import AuthResource, AsyncAuthResource
from arcadepy.types.auth_authorize_params import AuthRequirement, AuthRequirementOauth2

parametrize_provider_type = pytest.mark.parametrize(
    "provider_type, expected_provider_type",
    [
        (None, "oauth2"),
        ("oauth2", "oauth2"),
        ("custom_type", "custom_type"),
    ],
)

parametrize_scopes = pytest.mark.parametrize(
    "scopes, expected_scopes",
    [
        (["scope1", "scope2"], ["scope1", "scope2"]),
        (None, []),
    ],
)


@pytest.fixture
def sync_auth_resource() -> AuthResource:
    client = Arcade(api_key="test")
    auth = AuthResource(client)
    return auth


@pytest.fixture
def async_auth_resource() -> AsyncAuthResource:
    client = AsyncArcade(api_key="test")
    auth = AsyncAuthResource(client)
    return auth


@parametrize_provider_type
@parametrize_scopes
def test_start_calls_authorize_with_correct_params(
    sync_auth_resource: AuthResource,
    provider_type: Optional[str],
    expected_provider_type: str,
    scopes: Optional[List[str]],
    expected_scopes: List[str],
) -> None:
    auth = sync_auth_resource
    auth.authorize = Mock(return_value=AuthorizationResponse(status="pending"))  # type: ignore

    user_id = "user_id"
    provider = "github"

    auth.start(user_id, provider, provider_type=provider_type, scopes=scopes)

    auth.authorize.assert_called_with(
        auth_requirement=AuthRequirement(
            provider_id=provider,
            provider_type=expected_provider_type,
            oauth2=AuthRequirementOauth2(scopes=expected_scopes),
        ),
        user_id=user_id,
    )


@pytest.mark.asyncio
@parametrize_provider_type
@parametrize_scopes
async def test_async_start_calls_authorize_with_correct_params(
    async_auth_resource: AsyncAuthResource,
    provider_type: Optional[str],
    expected_provider_type: str,
    scopes: Optional[List[str]],
    expected_scopes: List[str],
) -> None:
    auth = async_auth_resource
    auth.authorize = AsyncMock(return_value=AuthorizationResponse(status="pending"))  # type: ignore

    user_id = "user_id"
    provider = "github"

    await auth.start(user_id, provider, provider_type=provider_type, scopes=scopes)

    auth.authorize.assert_called_with(
        auth_requirement=AuthRequirement(
            provider_id=provider,
            provider_type=expected_provider_type,
            oauth2=AuthRequirementOauth2(scopes=expected_scopes),
        ),
        user_id=user_id,
    )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["WorkerHealthResponse"]


class WorkerHealthResponse(BaseModel):
    id: Optional[str] = None

    enabled: Optional[bool] = None

    healthy: Optional[bool] = None

    message: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncOffsetPage", "AsyncOffsetPage"]

_T = TypeVar("_T")


class SyncOffsetPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]
    total_count: Optional[int] = None
    offset: Optional[int] = None

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        offset = self.offset
        if offset is None:
            return None

        length = len(self._get_page_items())
        current_count = offset + length

        total_count = self.total_count
        if total_count is None:
            return None

        if current_count < total_count:
            return PageInfo(params={"offset": current_count})

        return None


class AsyncOffsetPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]
    total_count: Optional[int] = None
    offset: Optional[int] = None

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        offset = self.offset
        if offset is None:
            return None

        length = len(self._get_page_items())
        current_count = offset + length

        total_count = self.total_count
        if total_count is None:
            return None

        if current_count < total_count:
            return PageInfo(params={"offset": current_count})

        return None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from arcadepy import Arcade, AsyncArcade
from tests.utils import assert_matches_type
from arcadepy.types import (
    ToolDefinition,
    WorkerResponse,
    WorkerHealthResponse,
)
from arcadepy.pagination import SyncOffsetPage, AsyncOffsetPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWorkers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Arcade) -> None:
        worker = client.workers.create(
            id="id",
            enabled=True,
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Arcade) -> None:
        worker = client.workers.create(
            id="id",
            enabled=True,
            http={
                "retry": 0,
                "secret": "secret",
                "timeout": 1,
                "uri": "uri",
            },
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.create(
            id="id",
            enabled=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.create(
            id="id",
            enabled=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert_matches_type(WorkerResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_update(self, client: Arcade) -> None:
        worker = client.workers.update(
            id="id",
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Arcade) -> None:
        worker = client.workers.update(
            id="id",
            enabled=True,
            http={
                "retry": 0,
                "secret": "secret",
                "timeout": 1,
                "uri": "uri",
            },
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert_matches_type(WorkerResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.workers.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: Arcade) -> None:
        worker = client.workers.list()
        assert_matches_type(SyncOffsetPage[WorkerResponse], worker, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Arcade) -> None:
        worker = client.workers.list(
            limit=0,
            offset=0,
        )
        assert_matches_type(SyncOffsetPage[WorkerResponse], worker, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert_matches_type(SyncOffsetPage[WorkerResponse], worker, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert_matches_type(SyncOffsetPage[WorkerResponse], worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Arcade) -> None:
        worker = client.workers.delete(
            "id",
        )
        assert worker is None

    @parametrize
    def test_raw_response_delete(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert worker is None

    @parametrize
    def test_streaming_response_delete(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert worker is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.workers.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_get(self, client: Arcade) -> None:
        worker = client.workers.get(
            "id",
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.get(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.get(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert_matches_type(WorkerResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.workers.with_raw_response.get(
                "",
            )

    @parametrize
    def test_method_health(self, client: Arcade) -> None:
        worker = client.workers.health(
            "id",
        )
        assert_matches_type(WorkerHealthResponse, worker, path=["response"])

    @parametrize
    def test_raw_response_health(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.health(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert_matches_type(WorkerHealthResponse, worker, path=["response"])

    @parametrize
    def test_streaming_response_health(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.health(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert_matches_type(WorkerHealthResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_health(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.workers.with_raw_response.health(
                "",
            )

    @parametrize
    def test_method_tools(self, client: Arcade) -> None:
        worker = client.workers.tools(
            id="id",
        )
        assert_matches_type(SyncOffsetPage[ToolDefinition], worker, path=["response"])

    @parametrize
    def test_method_tools_with_all_params(self, client: Arcade) -> None:
        worker = client.workers.tools(
            id="id",
            limit=0,
            offset=0,
        )
        assert_matches_type(SyncOffsetPage[ToolDefinition], worker, path=["response"])

    @parametrize
    def test_raw_response_tools(self, client: Arcade) -> None:
        response = client.workers.with_raw_response.tools(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = response.parse()
        assert_matches_type(SyncOffsetPage[ToolDefinition], worker, path=["response"])

    @parametrize
    def test_streaming_response_tools(self, client: Arcade) -> None:
        with client.workers.with_streaming_response.tools(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = response.parse()
            assert_matches_type(SyncOffsetPage[ToolDefinition], worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_tools(self, client: Arcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.workers.with_raw_response.tools(
                id="",
            )


class TestAsyncWorkers:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.create(
            id="id",
            enabled=True,
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.create(
            id="id",
            enabled=True,
            http={
                "retry": 0,
                "secret": "secret",
                "timeout": 1,
                "uri": "uri",
            },
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.create(
            id="id",
            enabled=True,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.create(
            id="id",
            enabled=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert_matches_type(WorkerResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_update(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.update(
            id="id",
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.update(
            id="id",
            enabled=True,
            http={
                "retry": 0,
                "secret": "secret",
                "timeout": 1,
                "uri": "uri",
            },
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert_matches_type(WorkerResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.workers.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.list()
        assert_matches_type(AsyncOffsetPage[WorkerResponse], worker, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.list(
            limit=0,
            offset=0,
        )
        assert_matches_type(AsyncOffsetPage[WorkerResponse], worker, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert_matches_type(AsyncOffsetPage[WorkerResponse], worker, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert_matches_type(AsyncOffsetPage[WorkerResponse], worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.delete(
            "id",
        )
        assert worker is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert worker is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert worker is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.workers.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_get(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.get(
            "id",
        )
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.get(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert_matches_type(WorkerResponse, worker, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.get(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert_matches_type(WorkerResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.workers.with_raw_response.get(
                "",
            )

    @parametrize
    async def test_method_health(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.health(
            "id",
        )
        assert_matches_type(WorkerHealthResponse, worker, path=["response"])

    @parametrize
    async def test_raw_response_health(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.health(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert_matches_type(WorkerHealthResponse, worker, path=["response"])

    @parametrize
    async def test_streaming_response_health(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.health(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert_matches_type(WorkerHealthResponse, worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_health(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.workers.with_raw_response.health(
                "",
            )

    @parametrize
    async def test_method_tools(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.tools(
            id="id",
        )
        assert_matches_type(AsyncOffsetPage[ToolDefinition], worker, path=["response"])

    @parametrize
    async def test_method_tools_with_all_params(self, async_client: AsyncArcade) -> None:
        worker = await async_client.workers.tools(
            id="id",
            limit=0,
            offset=0,
        )
        assert_matches_type(AsyncOffsetPage[ToolDefinition], worker, path=["response"])

    @parametrize
    async def test_raw_response_tools(self, async_client: AsyncArcade) -> None:
        response = await async_client.workers.with_raw_response.tools(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        worker = await response.parse()
        assert_matches_type(AsyncOffsetPage[ToolDefinition], worker, path=["response"])

    @parametrize
    async def test_streaming_response_tools(self, async_client: AsyncArcade) -> None:
        async with async_client.workers.with_streaming_response.tools(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            worker = await response.parse()
            assert_matches_type(AsyncOffsetPage[ToolDefinition], worker, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_tools(self, async_client: AsyncArcade) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.workers.with_raw_response.tools(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["ToolExecuteParams"]


class ToolExecuteParams(TypedDict, total=False):
    tool_name: Required[str]

    input: Dict[str, object]
    """JSON input to the tool, if any"""

    run_at: str
    """The time at which the tool should be run (optional).

    If not provided, the tool is run immediately
    """

    tool_version: str
    """The tool version to use (optional). If not provided, any version is used"""

    user_id: str



from unittest.mock import Mock, AsyncMock

import pytest

from arcadepy._client import Arcade, AsyncArcade
from arcadepy.types.shared import AuthorizationResponse
from arcadepy.resources.auth import AuthResource, AsyncAuthResource


@pytest.fixture
def sync_auth_resource() -> AuthResource:
    client = Arcade(api_key="test")
    auth = AuthResource(client)
    return auth


@pytest.fixture
def async_auth_resource() -> AsyncAuthResource:
    client = AsyncArcade(api_key="test")
    auth = AsyncAuthResource(client)
    return auth


def test_wait_for_completion_calls_status_from_auth_response(sync_auth_resource: AuthResource) -> None:
    auth = sync_auth_resource
    auth.status = Mock(return_value=AuthorizationResponse(status="completed"))  # type: ignore

    auth_response = AuthorizationResponse(status="pending", id="auth_id123")

    auth.wait_for_completion(auth_response)

    auth.status.assert_called_with(id="auth_id123", wait=45)


def test_wait_for_completion_raises_value_error_for_empty_authorization_id(sync_auth_resource: AuthResource) -> None:
    auth = sync_auth_resource
    auth_response = AuthorizationResponse(status="pending", id="", scopes=["scope1"])

    with pytest.raises(ValueError, match="Authorization ID is required"):
        auth.wait_for_completion(auth_response)


def test_wait_for_completion_calls_status_with_auth_id(sync_auth_resource: AuthResource) -> None:
    auth = sync_auth_resource
    auth.status = Mock(return_value=AuthorizationResponse(status="completed"))  # type: ignore

    auth.wait_for_completion("auth_id456")

    auth.status.assert_called_with(id="auth_id456", wait=45)


@pytest.mark.asyncio
async def test_async_wait_for_completion_calls_status_from_auth_response(
    async_auth_resource: AsyncAuthResource,
) -> None:
    auth = async_auth_resource
    auth.status = AsyncMock(return_value=AuthorizationResponse(status="completed"))  # type: ignore

    auth_response = AuthorizationResponse(status="pending", id="auth_id789")

    await auth.wait_for_completion(auth_response)

    auth.status.assert_called_with(id="auth_id789", wait=45)


@pytest.mark.asyncio
async def test_async_wait_for_completion_raises_value_error_for_empty_authorization_id(
    async_auth_resource: AsyncAuthResource,
) -> None:
    auth = async_auth_resource
    auth_response = AuthorizationResponse(status="pending", id="", scopes=["scope1"])

    with pytest.raises(ValueError, match="Authorization ID is required"):
        await auth.wait_for_completion(auth_response)


@pytest.mark.asyncio
async def test_async_wait_for_completion_calls_status_with_auth_id(async_auth_resource: AsyncAuthResource) -> None:
    auth = async_auth_resource
    auth.status = AsyncMock(return_value=AuthorizationResponse(status="completed"))  # type: ignore

    await auth.wait_for_completion("auth_id321")

    auth.status.assert_called_with(id="auth_id321", wait=45)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["WorkerResponse", "HTTP", "HTTPSecret"]


class HTTPSecret(BaseModel):
    binding: Optional[Literal["static", "tenant", "organization", "account"]] = None

    editable: Optional[bool] = None

    exists: Optional[bool] = None

    hint: Optional[str] = None

    value: Optional[str] = None


class HTTP(BaseModel):
    retry: Optional[int] = None

    secret: Optional[HTTPSecret] = None

    timeout: Optional[int] = None

    uri: Optional[str] = None


class WorkerResponse(BaseModel):
    id: Optional[str] = None

    enabled: Optional[bool] = None

    http: Optional[HTTP] = None

    type: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .usage import Usage as Usage
from .choice import Choice as Choice
from .shared import (
    Error as Error,
    AuthorizationContext as AuthorizationContext,
    AuthorizationResponse as AuthorizationResponse,
)
from .chat_message import ChatMessage as ChatMessage
from .value_schema import ValueSchema as ValueSchema
from .chat_response import ChatResponse as ChatResponse
from .health_schema import HealthSchema as HealthSchema
from .tool_execution import ToolExecution as ToolExecution
from .tool_definition import ToolDefinition as ToolDefinition
from .worker_response import WorkerResponse as WorkerResponse
from .tool_list_params import ToolListParams as ToolListParams
from .tool_get_response import ToolGetResponse as ToolGetResponse
from .auth_status_params import AuthStatusParams as AuthStatusParams
from .chat_message_param import ChatMessageParam as ChatMessageParam
from .worker_list_params import WorkerListParams as WorkerListParams
from .tool_execute_params import ToolExecuteParams as ToolExecuteParams
from .worker_tools_params import WorkerToolsParams as WorkerToolsParams
from .worker_create_params import WorkerCreateParams as WorkerCreateParams
from .worker_update_params import WorkerUpdateParams as WorkerUpdateParams
from .auth_authorize_params import AuthAuthorizeParams as AuthAuthorizeParams
from .execute_tool_response import ExecuteToolResponse as ExecuteToolResponse
from .tool_authorize_params import ToolAuthorizeParams as ToolAuthorizeParams
from .tool_execution_attempt import ToolExecutionAttempt as ToolExecutionAttempt
from .worker_health_response import WorkerHealthResponse as WorkerHealthResponse
